@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: kronolith/attendees.php,v 1.7.8.4 2005/07/18 19:24:27 selsky Exp $
+ * $Horde: kronolith/attendees.php,v 1.7.8.10 2006/05/22 09:52:09 jan Exp $
  *
- * Copyright 2004-2005 Code Fusion  <http://www.codefusion.co.za/>
+ * Copyright 2004-2006 Code Fusion  <http://www.codefusion.co.za/>
  *                Stuart Binge <s.binge@codefusion.co.za>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
@@ -20,37 +20,33 @@ require_once 'Horde/Variables.php';
 $attendees = (isset($_SESSION['attendees']) && is_array($_SESSION['attendees'])) ? $_SESSION['attendees'] : array();
 
 // Get our Action ID & Value. This specifies what action the user initiated.
-$actionID = Util::getFormData('actionID', false);
-$actionValue = Util::getFormData('actionValue', false);
-if (!$actionID) {
-    $actionID = Util::getFormData('clearAll') ? 'clear' : false;
-    if (!$actionID) {
-        $actionID = Util::getFormData('addNew') ? 'add' : false;
-        $actionValue = Util::getFormData('newAttendees', '');
-    }
+$actionID = Util::getFormData('actionID');
+if (Util::getFormData('clearAll')) {
+    $actionID =  'clear';
 }
-
-// Get the current Free/Busy view; default to the 'day' view if none specified.
-$view = Util::getFormData('view', 'day');
-
-// Pre-format our delete image/link.
-$delimg = Horde::img('delete.png', _("Remove Attendee"), null, $registry->getImageDir('horde'));
+$actionValue = Util::getFormData('actionValue');
 
 // Perform the specified action, if there is one.
 switch ($actionID) {
 case 'add':
     // Add new attendees. Multiple attendees can be seperated on a
     // single line by whitespace and/or commas.
-    $new = preg_split('/[\s,]+/', $actionValue, -1, PREG_SPLIT_NO_EMPTY);
+    $new = preg_split('/[\s,]+/', Util::getFormData('newAttendees'), -1, PREG_SPLIT_NO_EMPTY);
     if (is_array($new)) {
         foreach ($new as $newattendee) {
             // Avoid overwriting existing attendees with the default values
-            if (!array_key_exists($newattendee, $attendees))
+            if (!isset($attendees[$newattendee]))
                 $attendees[$newattendee] = array('attendance' => KRONOLITH_PART_REQUIRED, 'response' => KRONOLITH_RESPONSE_NONE);
         }
     }
 
     $_SESSION['attendees'] = $attendees;
+
+    if (Util::getFormData('addNewClose')) {
+        Util::closeWindowJS();
+        exit;
+    }
+
     break;
 
 case 'remove':
@@ -64,7 +60,7 @@ case 'remove':
 case 'changeatt':
     // Change the attendance status of an attendee
     list($partval, $partname) = explode(' ', $actionValue, 2);
-    if (array_key_exists($partname, $attendees)) {
+    if (isset($attendees[$partname])) {
         $attendees[$partname]['attendance'] = $partval;
         $_SESSION['attendees'] = $attendees;
     }
@@ -92,6 +88,10 @@ case 'dismiss':
     }
     break;
 
+case 'expand':
+    // Not implemented yet.
+    break;
+
 case 'clear':
     // Remove all the attendees.
     $attendees = array();
@@ -99,6 +99,12 @@ case 'clear':
     break;
 }
 
+// Get the current Free/Busy view; default to the 'day' view if none specified.
+$view = Util::getFormData('view', 'day');
+
+// Pre-format our delete image/link.
+$delimg = Horde::img('delete.png', _("Remove Attendee"), null, $registry->getImageDir('horde'));
+
 $ident = &Identity::singleton();
 $identities = $ident->getAll('id');
 $vars = &Variables::getDefaultVariables();
@@ -109,16 +115,26 @@ $tabs->addTab(_("Week"), 'javascript:switchView(\'week\')', 'week');
 
 $attendee_view = &Kronolith_FreeBusy_View::singleton($view);
 
-// Add the creator as a required attendee in the Free/Busy display (if
-// the user has an email address configured).
-$myemail = $ident->getValue('from_addr');
-if (!empty($myemail)) {
-    $vfb = Kronolith::getFreeBusy($myemail);
-    if (!is_a($vfb, 'PEAR_Error')) {
-        $attendee_view->addRequiredMember($vfb);
-    } else {
-        $notification->push(sprintf(_("Error retrieving free/busy information for %s: %s"), $myemail, $vfb->getMessage()));
+// Add the creator as a required attendee in the Free/Busy display
+$cal = @unserialize($prefs->getValue('fb_cals'));
+if (!is_array($cal)) {
+    $cal = null;
+}
+
+// If the free/busy calendars preference is empty, default to the user's
+// default_share preference, and if that's empty, to their username.
+if (!$cal) {
+    $cal = $prefs->getValue('default_share');
+    if (!$cal) {
+        $cal = Auth::getAuth();
     }
+    $cal = array($cal);
+}
+$vfb = Kronolith::generateFreeBusy($cal, null, null, true);
+if (!is_a($vfb, 'PEAR_Error')) {
+    $attendee_view->addRequiredMember($vfb);
+} else {
+    $notification->push(sprintf(_("Error retrieving your free/busy information: %s"), $vfb->getMessage()));
 }
 
 // Add the Free/Busy information for each attendee.
@@ -141,11 +157,10 @@ foreach ($attendees as $email => $status) {
 }
 
 $timestamp = (int)Util::getFormData('timestamp', time());
-list($vfb_html, $legend_html) = $attendee_view->render($timestamp);
+$vfb_html = $attendee_view->render($timestamp);
 
 $title = _("Edit attendees");
-$form_handler = Horde::applicationUrl('attendees.php');
-Horde::addScriptFile('tooltip.js', 'horde', true);
 require KRONOLITH_TEMPLATES . '/common-header.inc';
+$notification->notify(array('status'));
 require KRONOLITH_TEMPLATES . '/attendees/attendees.inc';
 require $registry->get('templates', 'horde') . '/common-footer.inc';
