@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: kronolith/data.php,v 1.72.2.1 2005/01/03 12:25:39 jan Exp $
+ * $Horde: kronolith/data.php,v 1.72.2.9 2006/06/16 11:33:12 jan Exp $
  *
- * Copyright 2001-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -26,16 +26,25 @@ if (!$conf['menu']['import_export']) {
 
 /* Importable file types. */
 $file_types = array('csv'       => _("Comma separated values"),
-                    'pdb'       => _("Palm DateBook"),
                     'icalendar' => _("vCalendar/iCalendar"));
 
 /* Templates for the different import steps. */
 $templates = array(
-    IMPORT_FILE => array(KRONOLITH_TEMPLATES . '/data/import.inc', KRONOLITH_TEMPLATES . '/data/export.inc'),
     IMPORT_CSV => array($registry->get('templates', 'horde') . '/data/csvinfo.inc'),
     IMPORT_MAPPED => array($registry->get('templates', 'horde') . '/data/csvmap.inc'),
     IMPORT_DATETIME => array($registry->get('templates', 'horde') . '/data/datemap.inc')
 );
+if (Kronolith::hasPermission('max_events') !== true &&
+    Kronolith::hasPermission('max_events') <= Kronolith::countEvents()) {
+    $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d events."), Kronolith::hasPermission('max_events')), ENT_COMPAT, NLS::getCharset());
+    if (!empty($conf['hooks']['permsdenied'])) {
+        $message = Horde::callHook('_perms_hook_denied', array('kronolith:max_events'), 'horde', $message);
+    }
+    $notification->push($message, 'horde.warning', array('content.raw'));
+    $templates[IMPORT_FILE] = array(KRONOLITH_TEMPLATES . '/data/export.inc');
+} else {
+    $templates[IMPORT_FILE] = array(KRONOLITH_TEMPLATES . '/data/import.inc', KRONOLITH_TEMPLATES . '/data/export.inc');
+}
 
 /* Initial values. */
 $import_step   = Util::getFormData('import_step', 0) + 1;
@@ -83,7 +92,11 @@ case 'export':
     }
 
     $events = array();
-    foreach ($display_calendars as $cal) {
+    $calendars = Util::getFormData('exportCal', $display_calendars);
+    if (!is_array($calendars)) {
+        $calendars = array($calendars);
+    }
+    foreach ($calendars as $cal) {
         if ($kronolith->getCalendar() != $cal) {
             $kronolith->close();
             $kronolith->open($cal);
@@ -164,6 +177,7 @@ case 'export':
 
 case IMPORT_FILE:
     $_SESSION['import_data']['import_cal'] = Util::getFormData('importCal');
+    $_SESSION['import_data']['purge'] = Util::getFormData('purge');
     break;
 }
 
@@ -174,20 +188,18 @@ if (!$error) {
         $next_step = IMPORT_FILE;
     } else {
         if ($actionID == IMPORT_FILE) {
-            if (!array_key_exists($_SESSION['import_data']['import_cal'], Kronolith::listCalendars())) {
+            $share = &$kronolith_shares->getShare($_SESSION['import_data']['import_cal']);
+            if (is_a($share, 'PEAR_Error')) {
                 $notification->push(_("You have specified an invalid calendar."), 'horde.error');
                 $next_step = $data->cleanup();
+            } elseif (!$share->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
+                $notification->push(_("You do not have permission to add events to the selected calendar."), 'horde.error');
+                $next_step = $data->cleanup();
             } else {
-                $share = &$kronolith_shares->getShare($_SESSION['import_data']['import_cal']);
-                if (!$share->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
-                    $notification->push(_("You do not have permission to add events to the selected calendar."), 'horde.error');
+                $next_step = $data->nextStep($actionID, $param);
+                if (is_a($next_step, 'PEAR_Error')) {
+                    $notification->push($next_step->getMessage(), 'horde.error');
                     $next_step = $data->cleanup();
-                } else {
-                    $next_step = $data->nextStep($actionID, $param);
-                    if (is_a($next_step, 'PEAR_Error')) {
-                        $notification->push($next_step->getMessage(), 'horde.error');
-                        $next_step = $data->cleanup();
-                    }
                 }
             }
         } else {
@@ -203,9 +215,32 @@ if (!$error) {
 /* We have a final result set. */
 if (is_array($next_step)) {
     $events = array();
-    $kronolith->open($_SESSION['import_data']['import_cal']);
     $error = false;
+    $max_events = Kronolith::hasPermission('max_events');
+    if ($max_events !== true) {
+        $num_events = Kronolith::countEvents();
+    }
+    $kronolith->open($_SESSION['import_data']['import_cal']);
+
+    /* Purge old calendar if requested. */
+    if ($_SESSION['import_data']['purge']) {
+        $result = $kronolith->delete($_SESSION['import_data']['import_cal']);
+        if (is_a($result, 'PEAR_Error')) {
+            $notification->push(sprintf(_("The calendar could not be purged: %s"), $result->getMessage()), 'horde.error');
+        } else {
+            $notification->push(_("Calendar successfully purged."), 'horde.success');
+        }
+    }
+
     foreach ($next_step as $row) {
+        if ($max_events !== true && $num_events >= $max_events) {
+            $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d events."), Kronolith::hasPermission('max_events')), ENT_COMPAT, NLS::getCharset());
+            if (!empty($conf['hooks']['permsdenied'])) {
+                $message = Horde::callHook('_perms_hook_denied', array('kronolith:max_events'), 'horde', $message);
+            }
+            $notification->push($message, 'horde.error', array('content.raw'));
+            break;
+        }
         $event = &$kronolith->getEvent();
         if (!$event || is_a($event, 'PEAR_Error')) {
             $msg = _("Can't create a new event.");
@@ -222,7 +257,12 @@ if (is_array($next_step)) {
             // Skip other iCalendar components for now.
             continue;
         } else {
-            $event->fromHash($row);
+            $valid = $event->fromHash($row);
+            if (is_a($valid, 'PEAR_Error')) {
+                $notification->push($valid, 'horde.error');
+                $error = true;
+                break;
+            }
         }
 
         $success = $event->save();
@@ -231,6 +271,10 @@ if (is_array($next_step)) {
             $error = true;
             break;
         }
+
+        if ($max_events !== true) {
+            $num_events++;
+        }
     }
     if (!$error) {
         $notification->push(sprintf(_("%s file successfully imported"),
@@ -239,51 +283,14 @@ if (is_array($next_step)) {
     $next_step = $data->cleanup();
 }
 
-// FIXME: Port PDB import code to new framework.
-/*
-    } elseif ($importID == IMPORT_PDB) {
-        $pdb = &Horde_Data::singleton('pdb');
-        $importData = $pdb->importFile($_FILES['import_file']['tmp_name']);
-
-        if (!isset($importData) || !is_array($importData)) {
-            $notification->push(_("There was an error importing the uploaded file"), 'horde.error');
-            $error = true;
-        } else {
-            $format = array('start_date' => 'year/month/mday',
-                            'end_date'   => 'year/month/mday',
-                            'start_time' => '24hr',
-                            'end_time'   => '24hr');
-            $delimiter = array('start_date' => '-',
-                               'end_date'   => '-',
-                               'start_time' => ':',
-                               'end_time'   => ':');
-
-            $map = array('title'          => 'title',
-                         'start_date'     => 'start_date',
-                         'start_time'     => 'start_time',
-                         'end_date'       => 'end_date',
-                         'end_time'       => 'end_time',
-                         'alarm'          => 'alarm',
-                         'alarm_date'     => 'alarm_date',
-                         'alarm_time'     => 'alarm_time',
-                         'description'    => 'description',
-                         'category'       => 'category',
-                         'location'       => 'location',
-                         'keywords'       => 'keywords',
-                         'recur_type'     => 'recur_type',
-                         'recur_end_date' => 'recur_end_date',
-                         'recur_interval' => 'recur_interval',
-                         'recur_data'     => 'recur_data');
-        }
-    }
-*/
-
 $title = _("Import/Export Calendar");
 require KRONOLITH_TEMPLATES . '/common-header.inc';
 require KRONOLITH_TEMPLATES . '/menu.inc';
 
+echo '<div id="page">';
 foreach ($templates[$next_step] as $template) {
     require $template;
 }
+echo '</div>';
 
 require $registry->get('templates', 'horde') . '/common-footer.inc';
