@@ -3,7 +3,7 @@
  * The Kronolith_Driver_sql:: class implements the Kronolith_Driver
  * API for a SQL backend.
  *
- * $Horde: kronolith/lib/Driver/sql.php,v 1.136 2004/12/21 16:41:44 chuck Exp $
+ * $Horde: kronolith/lib/Driver/sql.php,v 1.136.2.20 2006/05/03 21:54:29 jan Exp $
  *
  * @author  Luc Saillard <luc.saillard@fr.alcove.com>
  * @author  Chuck Hagenbuch <chuck@horde.org>
@@ -15,23 +15,23 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
     /**
      * The object handle for the current database connection.
      *
-     * @var object DB $_db
+     * @var DB
      */
     var $_db;
 
     /**
-     * Boolean indicating whether or not we're currently connected to
-     * the SQL server.
+     * Boolean indicating whether or not we're currently connected to the SQL
+     * server.
      *
-     * @var boolean $_connected
+     * @var boolean
      */
     var $_connected = false;
 
     /**
-     * Cache events as we fetch them to avoid fetching the same event
-     * from the DB twice.
+     * Cache events as we fetch them to avoid fetching the same event from the
+     * DB twice.
      *
-     * @var array $_cache
+     * @var array
      */
     var $_cache = array();
 
@@ -43,7 +43,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
 
     function listAlarms($date)
     {
-        $allevents = $this->listEvents($date, $date, true);
+        $allevents = $this->listEvents($date, null, true);
         $events = array();
 
         foreach ($allevents as $eventId) {
@@ -62,13 +62,14 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
                     $start = &new Horde_Date($next);
                     $start->min -= $event->getAlarm();
                     $start->correct();
+                    $end = &new Horde_Date(array('year' => $next->year,
+                                                 'month' => $next->month,
+                                                 'mday' => $next->mday,
+                                                 'hour' => $event->end->hour,
+                                                 'min' => $event->end->min,
+                                                 'sec' => $event->end->sec));
                     if ($start->compareDateTime($date) <= 0 &&
-                        $date->compareDateTime(new Horde_Date(array('year' => $next->year,
-                                                                    'month' => $next->month,
-                                                                    'mday' => $next->mday,
-                                                                    'hour' => $event->end->hour,
-                                                                    'min' => $event->end->min,
-                                                                    'sec' => $event->end->sec)) <= -1)) {
+                        $date->compareDateTime($end) <= -1) {
                         $events[] = $eventId;
                     }
                 }
@@ -80,23 +81,54 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
 
     function search($query)
     {
-        /* build SQL conditions based on the query string */
+        // Build SQL conditions based on the query string.
         $cond = '((';
+        $values = array();
 
         if (!empty($query->title)) {
-            $cond .= Horde_SQL::buildClause($this->_db, 'event_title', 'LIKE', $query->title) . ' AND ';
+            $binds = Horde_SQL::buildClause($this->_db, 'event_title', 'LIKE', $query->title, true);
+            if (is_array($binds)) {
+                $cond .= $binds[0] . ' AND ';
+                $values = array_merge($values, $binds[1]);
+            } else {
+                $cond .= $binds;
+            }
         }
         if (!empty($query->location)) {
-            $cond .= Horde_SQL::buildClause($this->_db, 'event_location', 'LIKE', $query->location) . ' AND ';
+            $binds = Horde_SQL::buildClause($this->_db, 'event_location', 'LIKE', $query->location, true);
+            if (is_array($binds)) {
+                $cond .= $binds[0] . ' AND ';
+                $values = array_merge($values, $binds[1]);
+            } else {
+                $cond .= $binds;
+            }
         }
         if (!empty($query->description)) {
-            $cond .= Horde_SQL::buildClause($this->_db, 'event_description', 'LIKE', $query->description) . ' AND ';
+            $binds = Horde_SQL::buildClause($this->_db, 'event_description', 'LIKE', $query->description, true);
+            if (is_array($binds)) {
+                $cond .= $binds[0] . ' AND ';
+                $values = array_merge($values, $binds[1]);
+            } else {
+                $cond .= $binds;
+            }
         }
         if (isset($query->category)) {
-            $cond .= Horde_SQL::buildClause($this->_db, 'event_category', '=', $query->category) . ' AND ';
+            $binds = Horde_SQL::buildClause($this->_db, 'event_category', '=', $query->category, true);
+            if (is_array($binds)) {
+                $cond .= $binds[0] . ' AND ';
+                $values = array_merge($values, $binds[1]);
+            } else {
+                $cond .= $binds;
+            }
         }
         if (isset($query->status)) {
-            $cond .= Horde_SQL::buildClause($this->_db, 'event_status', '=', $query->status) . ' AND ';
+            $binds = Horde_SQL::buildClause($this->_db, 'event_status', '=', $query->status, true);
+            if (is_array($binds)) {
+                $cond .= $binds[0] . ' AND ';
+                $values = array_merge($values, $binds[1]);
+            } else {
+                $cond .= $binds;
+            }
         }
 
         if ($cond == '((') {
@@ -105,11 +137,12 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             $cond = substr($cond, 0, strlen($cond) - 5) . '))';
         }
 
-        $eventIds = &$this->listEventsConditional($query->start,
-                                                  empty($query->end) ?
-                                                      new Horde_Date(array('mday' => 31, 'month' => 12, 'year' => 9999)) :
-                                                      $query->end,
-                                                  $cond);
+        $eventIds = $this->listEventsConditional($query->start,
+                                                 empty($query->end) ?
+                                                     new Horde_Date(array('mday' => 31, 'month' => 12, 'year' => 9999)) :
+                                                     $query->end,
+                                                 $cond,
+                                                 $values);
         if (is_a($eventIds, 'PEAR_Error')) {
             return $eventIds;
         }
@@ -126,6 +159,55 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
         return $events;
     }
 
+    /**
+     * Checks if the event's UID already exists and returns all event
+     * ids with that UID.
+     *
+     * @param string $uid          The event's uid.
+     * @param string $calendar_id  Calendar to search in.
+     *
+     * @return string|boolean  Returns a string with event_id or false if
+     *                         not found.
+     */
+    function exists($uid, $calendar_id = null)
+    {
+        $query = 'SELECT event_id  FROM ' . $this->_params['table'] . ' WHERE event_uid = ?';
+        $values = array($uid);
+
+        if (!is_null($calendar_id)) {
+            $query .= ' AND calendar_id = ?';
+            $values[] = $calendar_id;
+        }
+
+        // Log the query at a DEBUG log level.
+        Horde::logMessage(sprintf('SQL event fetch by %s: query = "%s"',
+                                  Auth::getAuth(), $query),
+                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        $event = &$this->_db->getRow($query, $values, DB_FETCHMODE_ASSOC);
+        if (is_a($event, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return $event;
+        }
+
+        if ($event) {
+            return $event['event_id'];
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Lists all events in the time range, optionally restricting
+     * results to only events with alarms.
+     *
+     * @param Horde_Date $startInterval  Start of range date object.
+     * @param Horde_Date $endInterval    End of range data object.
+     * @param boolean $hasAlarm          Only return events with alarms?
+     *                                   Defaults to all events.
+     *
+     * @return array  Events in the given time range.
+     */
     function listEvents($startDate = null, $endDate = null, $hasAlarm = false)
     {
         if (!isset($endDate)) {
@@ -150,72 +232,104 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
         }
 
         return $this->listEventsConditional($startInterval, $endInterval,
-                                            $hasAlarm ? 'event_alarm > 0' : '');
+                                            $hasAlarm ? 'event_alarm > ?' : '',
+                                            $hasAlarm ? array(0) : array());
     }
 
     /**
      * Lists all events that satisfy the given conditions.
      *
-     * @param object $startInterval  Start of range date object.
-     * @param object $endInterval    End of range data object.
-     * @param string $conditions  Conditions, given as SQL clauses.
+     * @param Horde_Date $startInterval  Start of range date object.
+     * @param Horde_Date $endInterval    End of range data object.
+     * @param string $conditions         Conditions, given as SQL clauses.
+     * @param array $vals                SQL bind variables for use with
+     *                                   $conditions clauses.
      *
-     * @return array  Events in the given time range satisfying the given conditions.
+     * @return array  Events in the given time range satisfying the given
+     *                conditions.
      */
-    function listEventsConditional($startInterval, $endInterval, $conditions = '')
+    function listEventsConditional($startInterval, $endInterval,
+                                   $conditions = '', $vals = array())
     {
         $q = 'SELECT event_id, event_uid, event_description, event_location,' .
             ' event_status, event_attendees,' .
             ' event_keywords, event_title, event_category,' .
             ' event_recurtype, event_recurenddate, event_recurinterval,' .
             ' event_recurdays, event_start, event_end, event_alarm,' .
-            ' event_modified, event_exceptions, event_creator_id FROM ' . $this->_params['table'] .
-            ' WHERE calendar_id = ' . $this->_db->quote($this->_calendar) . ' AND ((';
+            ' event_modified, event_exceptions, event_creator_id' .
+            ' FROM ' . $this->_params['table'] .
+            ' WHERE calendar_id = ? AND ((';
+        $values = array($this->_calendar);
 
         if ($conditions) {
             $q .= $conditions . ')) AND ((';
+            $values = array_merge($values, $vals);
         }
 
         $etime = sprintf('%04d-%02d-%02d 00:00:00', $endInterval->year, $endInterval->month, $endInterval->mday);
         if (isset($startInterval)) {
             $stime = sprintf('%04d-%02d-%02d 00:00:00', $startInterval->year, $startInterval->month, $startInterval->mday);
-            $q .= 'event_end > ' . $this->_db->quote($stime) . ' AND ';
+            $q .= 'event_end > ? AND ';
+            $values[] = $stime;
         }
-        $q .= 'event_start < ' . $this->_db->quote($etime) . ') OR (';
+        $q .= 'event_start < ?) OR (';
+        $values[] = $etime;
         if (isset($stime)) {
-            $q .= 'event_recurenddate >= ' . $this->_db->quote($stime) . ' AND ';
+            $q .= 'event_recurenddate >= ? AND ';
+            $values[] = $stime;
         }
-        $q .= 'event_start <= ' . $this->_db->quote($etime) .
-            ' AND event_recurtype != ' . KRONOLITH_RECUR_NONE . '))';
+        $q .= 'event_start <= ?' .
+            ' AND event_recurtype <> ?))';
+        array_push($values, $etime, KRONOLITH_RECUR_NONE);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL event list by %s: query = "%s"',
                                   Auth::getAuth(), $q),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        /* Run the query. */
-        $qr = $this->_db->query($q);
+        // Run the query.
+        $qr = $this->_db->query($q, $values);
+        if (is_a($qr, 'PEAR_Error')) {
+            Horde::logMessage($qr, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return array();
+        }
 
         $events = array();
-        if (!is_a($qr, 'PEAR_Error')) {
-            $row = $qr->fetchRow(DB_FETCHMODE_ASSOC);
-            while ($row && !is_a($row, 'PEAR_Error')) {
-                // We have all the information we need to create an
-                // event object for this event, so go ahead and cache
-                // it.
-                $this->_cache[$this->_calendar][$row['event_id']] = &new Kronolith_Event_sql($this, $row);
+        $row = $qr->fetchRow(DB_FETCHMODE_ASSOC);
+        while ($row && !is_a($row, 'PEAR_Error')) {
+            // If the event did not have a UID before, we need to give
+            // it one.
+            if (empty($row['event_uid'])) {
+                $row['event_uid'] = $this->generateUID();
 
-                if ($row['event_recurtype'] == KRONOLITH_RECUR_NONE) {
-                    $events[$row['event_id']] = $row['event_id'];
-                } else {
-                    $next = $this->nextRecurrence($row['event_id'], $startInterval);
-                    if ($next && $next->compareDate($endInterval) < 0) {
-                        $events[$row['event_id']] = $row['event_id'];
-                    }
+                // Save the new UID for data integrity.
+                $query = 'UPDATE ' . $this->_params['table'] . ' SET event_uid = ? WHERE event_id = ?';
+                $values = array($row['event_uid'], $row['event_id']);
+
+                // Log the query at a DEBUG log level.
+                Horde::logMessage(sprintf('SQL event uid autocreation for %s: query = "%s"',
+                                          Auth::getAuth(), $query),
+                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+                $result = $this->_db->query($query, $values);
+                if (is_a($result, 'PEAR_Error')) {
+                    Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                 }
-
-                $row = $qr->fetchRow(DB_FETCHMODE_ASSOC);
             }
+
+            // We have all the information we need to create an event
+            // object for this event, so go ahead and cache it.
+            $this->_cache[$this->_calendar][$row['event_id']] = &new Kronolith_Event_sql($this, $row);
+            if ($row['event_recurtype'] == KRONOLITH_RECUR_NONE) {
+                $events[$row['event_uid']] = $row['event_id'];
+            } else {
+                $next = $this->nextRecurrence($row['event_id'], $startInterval);
+                if ($next && $next->compareDate($endInterval) < 0) {
+                    $events[$row['event_uid']] = $row['event_id'];
+                }
+            }
+
+            $row = $qr->fetchRow(DB_FETCHMODE_ASSOC);
         }
 
         return $events;
@@ -224,7 +338,8 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
     function &getEvent($eventId = null)
     {
         if (is_null($eventId)) {
-            return $event = &new Kronolith_Event_sql($this);
+            $event = &new Kronolith_Event_sql($this);
+            return $event;
         }
 
         if (isset($this->_cache[$this->_calendar][$eventId])) {
@@ -237,16 +352,15 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             ' event_recurtype, event_recurenddate, event_recurinterval,' .
             ' event_recurdays, event_start, event_end, event_alarm,' .
             ' event_modified, event_exceptions, event_creator_id' .
-            ' FROM ' . $this->_params['table'] .
-            ' WHERE event_id = ' . $this->_db->quote($eventId) .
-            ' AND calendar_id = ' . $this->_db->quote($this->_calendar);
+            ' FROM ' . $this->_params['table'] . ' WHERE event_id = ? AND calendar_id = ?';
+        $values = array($eventId, $this->_calendar);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL event fetch by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $event = &$this->_db->getRow($query, DB_FETCHMODE_ASSOC);
+        $event = &$this->_db->getRow($query, $values, DB_FETCHMODE_ASSOC);
         if (is_a($event, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $event;
@@ -256,7 +370,7 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             $this->_cache[$this->_calendar][$eventId] = &new Kronolith_Event_sql($this, $event);
             return $this->_cache[$this->_calendar][$eventId];
         } else {
-            return false;
+            return PEAR::raiseError(_("Event not found"));
         }
     }
 
@@ -270,15 +384,15 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
             ' event_recurtype, event_recurenddate, event_recurinterval,' .
             ' event_recurdays, event_start, event_end, event_alarm,' .
             ' event_modified, event_exceptions, event_creator_id' .
-            ' FROM ' . $this->_params['table'] .
-            ' WHERE event_uid = ' . $this->_db->quote($uid);
+            ' FROM ' . $this->_params['table'] . ' WHERE event_uid = ?';
+        $values = array($uid);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL event fetch by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $event = &$this->_db->getRow($query, DB_FETCHMODE_ASSOC);
+        $event = &$this->_db->getRow($query, $values, DB_FETCHMODE_ASSOC);
         if (is_a($event, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $event;
@@ -293,78 +407,104 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
         }
     }
 
-    function saveEvent($event)
+    /**
+     * Saves an event in the backend.
+     * If it is a new event, it is added, otherwise the event is updated.
+     *
+     * @param Kronolith_Event $event  The event to save.
+     */
+    function saveEvent(&$event)
     {
-        if ($event->isStored()) {
+        if ($event->isStored() || $event->exists()) {
+            $values = array();
+
             $query = 'UPDATE ' . $this->_params['table'] . ' SET ';
 
             foreach ($event->getProperties() as $key => $val) {
-                $query .= " $key = " . $this->_db->quote($val) . ',';
+                $query .= " $key = ?,";
+                $values[] = $val;
             }
             $query = substr($query, 0, -1);
-            $query .= ' WHERE event_id = ' . $this->_db->quote($event->getId());
+            $query .= ' WHERE event_id = ?';
+            $values[] = $event->getId();
 
-            /* Log the query at a DEBUG log level. */
+            // Log the query at a DEBUG log level.
             Horde::logMessage(sprintf('SQL event update by %s: query = "%s"',
                                       Auth::getAuth(), $query),
                               __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-            $result = $this->_db->query($query);
+            $result = $this->_db->query($query, $values);
             if (is_a($result, 'PEAR_Error')) {
                 Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                 return $result;
             }
 
-            /* Log the modification of this item in the history
-             * log. */
+            // Log the modification of this item in the history log.
             if ($event->getUID()) {
                 $history = &Horde_History::singleton();
                 $history->log('kronolith:' . $this->_calendar . ':' . $event->getUID(), array('action' => 'modify'), true);
             }
 
+            // Notify users about the changed event.
+            $result = Kronolith::sendNotification($event, 'edit');
+            if (is_a($result, 'PEAR_Error')) {
+                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+
             return $event->getId();
         } else {
             if ($event->getId()) {
                 $id = $event->getId();
             } else {
                 $id = md5(uniqid(mt_rand(), true));
+                $event->setId($id);
             }
 
             if ($event->getUID()) {
                 $uid = $event->getUID();
             } else {
                 $uid = $this->generateUID();
+                $event->setUID($uid);
             }
 
-            $query = 'INSERT INTO ' . $this->_params['table'] . ' ';
-            $cols_name = '(event_id,event_uid,';
-            $cols_values = 'values (' . $this->_db->quote($id) . ',' . $this->_db->quote($uid) . ',';
+            $query = 'INSERT INTO ' . $this->_params['table'];
+            $cols_name = ' (event_id, event_uid,';
+            $cols_values = ' VALUES (?, ?,';
+            $values = array($id, $uid);
 
             foreach ($event->getProperties() as $key => $val) {
                 $cols_name .= " $key,";
-                $cols_values .= $this->_db->quote($val) . ',';
+                $cols_values .= ' ?,';
+                $values[] = $val;
             }
 
             $cols_name .= ' calendar_id)';
-            $cols_values .= $this->_db->quote($this->_calendar) . ') ';
+            $cols_values .= ' ?)';
+            $values[] = $this->_calendar;
 
             $query .= $cols_name . $cols_values;
 
-            /* Log the query at a DEBUG log level. */
+            // Log the query at a DEBUG log level.
             Horde::logMessage(sprintf('SQL event store by %s: query = "%s"',
                                 Auth::getAuth(), $query),
                                 __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-            $result = $this->_db->query($query);
+            $result = $this->_db->query($query, $values);
             if (is_a($result, 'PEAR_Error')) {
                 Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                 return $result;
             }
 
-            /* Log the creation of this item in the history log. */
+            // Log the creation of this item in the history log.
             $history = &Horde_History::singleton();
             $history->log('kronolith:' . $this->_calendar . ':' . $uid, array('action' => 'add'), true);
 
+            // Notify users about the new event.
+            $result = Kronolith::sendNotification($event, 'add');
+            if (is_a($result, 'PEAR_Error')) {
+                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+
             return $id;
         }
     }
@@ -377,21 +517,18 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
      */
     function move($eventId, $newCalendar)
     {
-        /* Make sure we have a valid database connection. */
+        // Make sure we have a valid database connection.
         $this->_connect();
 
-        $query = sprintf('UPDATE %s SET calendar_id = %s WHERE calendar_id = %s AND event_id = %s',
-                         $this->_params['table'],
-                         $this->_db->quote($newCalendar),
-                         $this->_db->quote($this->_calendar),
-                         $this->_db->quote($eventId));
+        $query = 'UPDATE ' . $this->_params['table'] . ' SET calendar_id = ? WHERE calendar_id = ? AND event_id = ?';
+        $values = array($newCalendar, $this->_calendar, $eventId);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('Kronolith_Driver_sql::move(): %s', $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        /* Attempt the move query. */
-        $result = $this->_db->query($query);
+        // Attempt the move query.
+        $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -411,16 +548,15 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
     {
         $this->_connect();
 
-        $query = sprintf('DELETE FROM %s WHERE calendar_id = %s',
-                         $this->_params['table'],
-                         $this->_db->quote($calendar));
+        $query = 'DELETE FROM ' . $this->_params['table'] . ' WHERE calendar_id = ?';
+        $values = array($calendar);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL Calender Delete by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        return $this->_db->query($query);
+        return $this->_db->query($query, $values);
     }
 
     /**
@@ -432,31 +568,35 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
      */
     function deleteEvent($eventId)
     {
-        /* Fetch the event for later use. */
+        // Fetch the event for later use.
         $event = &$this->getEvent($eventId);
 
-        $query = sprintf('DELETE FROM %s WHERE event_id = %s AND calendar_id = %s',
-                         $this->_params['table'],
-                         $this->_db->quote($eventId),
-                         $this->_db->quote($this->_calendar));
+        $query = 'DELETE FROM ' . $this->_params['table'] . ' WHERE event_id = ? AND calendar_id = ?';
+        $values = array($eventId, $this->_calendar);
 
-        /* Log the query at a DEBUG log level. */
+        // Log the query at a DEBUG log level.
         Horde::logMessage(sprintf('SQL Event Delete by %s: query = "%s"',
                                   Auth::getAuth(), $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        $result = $this->_db->query($query);
+        $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
 
-        /* Log the deletion of this item in the history log. */
+        // Log the deletion of this item in the history log.
         if ($event->getUID()) {
             $history = &Horde_History::singleton();
             $history->log('kronolith:' . $this->_calendar . ':' . $event->getUID(), array('action' => 'delete'), true);
         }
 
+        // Notify about the deleted event.
+        $result = Kronolith::sendNotification($event, 'delete');
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
         return true;
     }
 
@@ -469,13 +609,22 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
     {
         if (!$this->_connected) {
             Horde::assertDriverConfig($this->_params, 'calendar',
-                array('phptype', 'hostspec', 'username', 'database'));
+                array('phptype'));
 
+            if (!isset($this->_params['database'])) {
+                $this->_params['database'] = '';
+            }
+            if (!isset($this->_params['username'])) {
+                $this->_params['username'] = '';
+            }
+            if (!isset($this->_params['hostspec'])) {
+                $this->_params['hostspec'] = '';
+            }
             if (!isset($this->_params['table'])) {
                 $this->_params['table'] = 'kronolith_events';
             }
 
-            /* Connect to the SQL server using the supplied parameters. */
+            // Connect to the SQL server using the supplied parameters.
             require_once 'DB.php';
             $this->_db = &DB::connect($this->_params,
                                       array('persistent' => !empty($this->_params['persistent'])));
@@ -483,17 +632,35 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
                 Horde::fatal($this->_db, __FILE__, __LINE__);
             }
 
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            // Set DB portability options.
+            switch ($this->_db->phptype) {
+            case 'mssql':
+                $this->_db->_setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
 
             $this->_connected = true;
 
-            /* Handle any database specific initialization code to
-             * run. */
+            // Handle any database specific initialization code to
+            // run.
             switch ($this->_db->dbsyntax) {
             case 'oci8':
                 $query = "ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'";
 
-                /* Log the query at a DEBUG log level. */
+                // Log the query at a DEBUG log level.
+                Horde::logMessage(sprintf('SQL session setup by %s: query = "%s"',
+                                          Auth::getAuth(), $query),
+                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+                $this->_db->query($query);
+                break;
+
+            case 'pgsql':
+                $query = "SET datestyle TO 'iso'";
+
+                // Log the query at a DEBUG log level.
                 Horde::logMessage(sprintf('SQL session setup by %s: query = "%s"',
                                           Auth::getAuth(), $query),
                                   __FILE__, __LINE__, PEAR_LOG_DEBUG);
@@ -544,6 +711,9 @@ class Kronolith_Driver_sql extends Kronolith_Driver {
  */
 class Kronolith_Event_sql extends Kronolith_Event {
 
+    /**
+     * @var array
+     */
     var $_properties = array();
 
     function fromDriver($SQLEvent)
