@@ -5,12 +5,12 @@ require_once 'Horde/Template.php';
 /**
  * This class represent a view of multiple free busy information sets.
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2003-2006 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2004-2006 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information.
  *
- * $Horde: kronolith/lib/FBView.php,v 1.10.10.1 2005/01/03 12:25:40 jan Exp $
+ * $Horde: kronolith/lib/FBView.php,v 1.10.10.8 2006/01/01 21:29:03 jan Exp $
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
  * @author  Jan Schneider <jan@horde.org>
@@ -24,7 +24,6 @@ class Kronolith_FreeBusy_View {
     var $_startHour;
     var $_endHour;
 
-    var $_day;
     var $_startStamp;
     var $_endStamp;
 
@@ -43,7 +42,7 @@ class Kronolith_FreeBusy_View {
         global $prefs;
 
         $this->_startHour = floor($prefs->getValue('day_hour_start') / 2);
-        $this->_endHour = floor($prefs->getValue('day_hour_end') / 2);
+        $this->_endHour = floor(($prefs->getValue('day_hour_end') + 1) / 2);
 
         $this->_render($day);
 
@@ -73,10 +72,16 @@ class Kronolith_FreeBusy_View {
         $template = &new Horde_Template();
         $template->set('title', $this->_title());
 
-        $html = $template->fetch(KRONOLITH_TEMPLATES . '/fbview/header.tpl');
+        $html = $template->fetch(KRONOLITH_TEMPLATES . '/fbview/header.tpl') .
+            '<div class="fbgrid">';
 
         $hours_html = $this->_hours();
 
+        // set C locale to avoid localized decimal separators during CSS width
+        // calculation.
+        $lc = setlocale(LC_NUMERIC, 0);
+        setlocale(LC_NUMERIC, 'C');
+
         // Required to attend.
         if (count($this->_requiredMembers) > 0) {
             $template = &new Horde_Template();
@@ -90,9 +95,11 @@ class Kronolith_FreeBusy_View {
             }
 
             $template = &new Horde_Template();
-            $template->set('title', _("Required to attend"));
+            $template->set('title', _("Required Attendees"));
             $template->set('rows', $rows);
+            $template->set('span', count($this->_timeBlocks));
             $template->set('hours', $hours_html);
+            $template->set('legend', '');
             $html .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl');
         }
 
@@ -109,13 +116,15 @@ class Kronolith_FreeBusy_View {
             }
 
             $template = &new Horde_Template();
-            $template->set('title', _("Optional to attend"));
+            $template->set('title', _("Optional Attendees"));
             $template->set('rows', $rows);
+            $template->set('span', count($this->_timeBlocks));
             $template->set('hours', $hours_html);
+            $template->set('legend', '');
             $html .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl');
         }
 
-        // possible meeting times.
+        // Possible meeting times.
         $optimal->setAttribute('ORGANIZER', _("All Attendees"));
         $blocks = $this->_getBlocks($optimal,
                                     $optimal->getFreePeriods($this->_startStamp, $this->_endStamp),
@@ -126,7 +135,7 @@ class Kronolith_FreeBusy_View {
         $template->set('blocks', $blocks);
         $rows = $template->fetch(KRONOLITH_TEMPLATES . '/fbview/row.tpl');
 
-        // possible meeting times.
+        // Possible meeting times.
         $required->setAttribute('ORGANIZER', _("Required Attendees"));
         $blocks = $this->_getBlocks($required,
                                     $required->getFreePeriods($this->_startStamp, $this->_endStamp),
@@ -137,19 +146,22 @@ class Kronolith_FreeBusy_View {
         $template->set('blocks', $blocks);
         $rows .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/row.tpl');
 
-        $template = &new Horde_Template();
-        $template->set('title', _("Possible Meeting Times"));
-        $template->set('rows', $rows);
-        $template->set('hours', $hours_html);
-        $html .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl');
+        // Reset locale.
+        setlocale(LC_NUMERIC, $lc);
 
+        $template = &new Horde_Template();
+        $template->set('rows', $rows);
+        $template->set('title', _("Overview"));
+        $template->set('span', count($this->_timeBlocks));
+        $template->set('hours', $hours_html);
         if ($prefs->getValue('show_fb_legend')) {
-            $legend_html = Util::bufferOutput('require' , KRONOLITH_TEMPLATES . '/fbview/legend.inc');
+            $template->setOption('gettext', true);
+            $template->set('legend', $template->fetch(KRONOLITH_TEMPLATES . '/fbview/legend.tpl'));
         } else {
-            $legend_html = '';
+            $template->set('legend', '');
         }
 
-        return array($html, $legend_html);
+        return $html . $template->fetch(KRONOLITH_TEMPLATES . '/fbview/section.tpl') . '</div>';
     }
 
     /**
@@ -168,10 +180,12 @@ class Kronolith_FreeBusy_View {
         require_once dirname(__FILE__) . '/FBView/' . $view . '.php';
         $class = 'Kronolith_FreeBusy_View_' . $view;
         if (class_exists($class)) {
-            return $ret = &new $class($user, $params);
+            $view = &new $class($user, $params);
         } else {
-            return false;
+            $view = false;
         }
+
+        return $view;
     }
 
     /**
@@ -204,4 +218,33 @@ class Kronolith_FreeBusy_View {
         return $instances[$view];
     }
 
+    function _getBlocks($member, $periods, $blockfile, $label)
+    {
+        $template = &new Horde_Template();
+        $template->set('label', $label);
+
+        reset($periods);
+        list($periodStart, $periodEnd) = each($periods);
+
+        $blocks = '';
+        foreach ($this->_timeBlocks as $start => $end) {
+            if ($member->getStart() > $start ||
+                $member->getEnd() < $end) {
+                $blocks .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/unknownblock.tpl');
+                continue;
+            }
+
+            while ($end > $periodEnd &&
+                   list($periodStart, $periodEnd) = each($periods));
+
+            if ($start <= $periodEnd && $end >= $periodStart) {
+                $blocks .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/' . $blockfile);
+            } else {
+                $blocks .= $template->fetch(KRONOLITH_TEMPLATES . '/fbview/emptyblock.tpl');
+            }
+        }
+
+        return $blocks;
+    }
+
 }
